using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using EAppointments.BMS.Client.DirectoryService;
using EAppointments.BMS.Services.Contracts;

namespace EAppointments.BMS.Client
{
    public enum UserType
    {
        Patient,
        Referrer
    }
    
    public class User
    {
        private Guid _id;
        private String _firstName;
        private String _lastName;
        private UserType _type;

        public Guid Id
        {
            get { return _id; }
        }

        public String Name
        {
            get { return String.Format("{0} {1}", _firstName, _lastName); }
        }

        public UserType Type
        {
            get { return _type; }
        }
  

        public User(Guid? id, UserType type)
        {
            DirectoryServiceClient directoryService = new DirectoryServiceClient();
            directoryService.ClientCredentials.UserName.UserName = Context.Current.LoginId;
            directoryService.ClientCredentials.UserName.Password = Context.Current.Password;
            _type = type;
            try
            {
                switch (type)
                {
                    case UserType.Patient:
                        {
                            PatientSearchCriteria patientCriteria = new PatientSearchCriteria();
                            patientCriteria.Id = id;
                            Patient patient = (Patient)directoryService.FindPatient(patientCriteria)[0];
                            this._id = patient.Id;
                            this._firstName = patient.FirstName;
                            this._lastName = patient.LastName;
                            break;
                        }
                    case UserType.Referrer:
                        {
                            ReferrerSearchCriteria referrerCriteria = new ReferrerSearchCriteria();
                            referrerCriteria.Id = id;
                            Referrer referrer = (Referrer)directoryService.FindReferrer(referrerCriteria)[0];
                            this._id = referrer.Id;
                            this._firstName = referrer.FirstName;
                            this._lastName = referrer.LastName;
                            break;
                        }
                }
            }
            catch (Exception exception)
            {
                ExceptionDialog.Show(exception);
            }
        }

    }    
    
    internal class Context
    {
        #region Singleton
        private static Context instance;

        public static Context Current
        {
            get { return instance; }
        }

        static Context()
        {
            instance = new Context();
        }
        #endregion

        private User _currentUser;

        public User CurrentUser
        {
            get 
            {
                if (_currentUser == null)
                {
                    SelectUser();
                    if (_currentUser == null)
                    {
                        Application.Exit();
                    }
                }
                return _currentUser;
            }
            set { _currentUser = value; }
        }

        public String LoginId
        {
            get { return "bmsadmin"; }
        }

        public String Password
        {
            get { return "password"; }
        }

        public static void SelectUser()
        {
            using (SelectUserDialog selectUserDialog = new SelectUserDialog())
            {
                selectUserDialog.ShowDialog();
                if (selectUserDialog.DialogResult == DialogResult.OK && selectUserDialog.SelectedUserId != null)
                {
                    Context.Current.CurrentUser = new User(selectUserDialog.SelectedUserId, selectUserDialog.SelectedUserType);
                }              
            }
        }
    }
}