using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using EAppointments.BMS.Services.Contracts;

namespace EAppointments.BMS.Client
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();           
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            cmbViews.SelectedIndex = 0;
            RefreshView();
        }

        private void RefreshView()
        {
            User user = Context.Current.CurrentUser;
            if (user != null)
            {
                this.lblUserName.Text = "Welcome BMS User";
                txtUser.Text = Context.Current.CurrentUser.Name;
                this.lstAppointments.User = Context.Current.CurrentUser;
                this.lstAppointments.SearchCriteria = BuildCriteria(user);
            }
        }

        private AppointmentSearchCriteria BuildCriteria(User user)
        {
            AppointmentSearchCriteria criteria = new AppointmentSearchCriteria();
           
            switch (user.Type)
            {
                case UserType.Patient:
                    criteria.PatientId = user.Id;
                    criteria.ReferrerId = null;
                    break;

                case UserType.Referrer:
                    criteria.ReferrerId = user.Id;
                    criteria.PatientId = null;
                    break;
            }

            switch (cmbViews.SelectedIndex)
            {
                case 0:  // All
                    criteria.Status = null;
                    break;
                case 1: // Future
                    criteria.StartDateTime = DateTime.Now;
                    break;
                case 2: // Pending
                    criteria.Status = (int)AppointmentStatus.Pending;
                    break;
                case 3: // Booked
                    criteria.Status = (int)AppointmentStatus.Booked;
                    break;
                case 4: // Approved
                    criteria.Status = (int)AppointmentStatus.Approved;
                    break;
                case 5: // Rejected
                    criteria.Status = (int)AppointmentStatus.Rejected;
                    break;
                case 6: // Past
                    criteria.EndDateTime = DateTime.Now;
                    break;
            }

            return criteria;
        }

        private void btnSelect_Click(object sender, EventArgs e)
        {
            Context.SelectUser();
            RefreshView();
        }

        private void cmbViews_SelectedIndexChanged(object sender, EventArgs e)
        {
            RefreshView();
        }

        private void lnkRefresh_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            RefreshView();
        }

        private void lnkCreateNew_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            MessageBox.Show("Not implemented in this release");
        }

        private void lnkEdit_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            MessageBox.Show("Not implemented in this release");
        }

        private void lnkChangeStatus_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            MessageBox.Show("Not implemented in this release");
        }      
    }
}