using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using EAppointments.BMS.Client.DirectoryService;
using EAppointments.BMS.Client.AppointmentService;
using EAppointments.BMS.Services.Contracts;

namespace EAppointments.BMS.Client
{
    public partial class AppointmentListControl : UserControl
    {
        DirectoryServiceClient directoryService = new DirectoryServiceClient();
        AppointmentServiceClient appointmentClient = new AppointmentServiceClient();
        
        private User _user;
        public User User
        {
            get { return _user; }
            set { _user = value; }
        }     

        private Appointment _selectedAppointment;
        public Appointment SelectedAppointment
        {
            get { return _selectedAppointment; }
            set { _selectedAppointment = value; }
        }

        private AppointmentSearchCriteria _searchCriteria;
        public AppointmentSearchCriteria SearchCriteria
        {
            get { return _searchCriteria; }
            set
            {
                _searchCriteria = value;
                GetAppointmentList();
            }
        }

        public event EventHandler SelectionChanged;
 
        public AppointmentListControl()
        {           
            
            InitializeComponent();
            directoryService.ClientCredentials.UserName.UserName = Context.Current.LoginId;
            directoryService.ClientCredentials.UserName.Password = Context.Current.Password;

            appointmentClient.ClientCredentials.UserName.UserName = Context.Current.LoginId;
            appointmentClient.ClientCredentials.UserName.Password = Context.Current.Password;
        }

        private void GetAppointmentList()
        {
            if (_user == null || _searchCriteria == null)
                return;
            try
            {
                EAppointments.BMS.Client.AppointmentService.AppointmentCollection appointments = appointmentClient.Find(_searchCriteria);

                lstAppointments.Items.Clear();
                if (appointments != null)
                {
                    foreach (Appointment appointment in appointments)
                    {
                        Referrer referrer = GetReferrer(appointment.Referrer.Id);
                        Patient patient = GetPatient(appointment.Patient.Id);

                        ListViewItem item = new ListViewItem(
                            new string[] { 
                            appointment.Ubrn.ToString(),
                            appointment.StartDateTime.ToString(),
                            appointment.Status.ToString(),
                            appointment.CreatedDateTime.Date.ToShortDateString(),                            
                            String.Format("{0} {1}", referrer.FirstName,referrer.LastName),
                           String.Format("{0} {1}", patient.FirstName,patient.LastName)              
                        });
                        item.Tag = appointment;
                        this.lstAppointments.Items.Add(item);
                    }
                }
            }
            catch (Exception exception)
            {
                ExceptionDialog.Show(exception);
            }


            if (lstAppointments.Items.Count > 0)
            {
                _selectedAppointment = lstAppointments.Items[0].Tag as Appointment;
                OnSelectionChanged(this, EventArgs.Empty);
            }
        }

        private Referrer GetReferrer(Guid referrerId)
        {  
            ReferrerSearchCriteria criteria = new ReferrerSearchCriteria();
            criteria.Id = referrerId;
            Referrer referrer = directoryService.FindReferrer(criteria)[0];
            return referrer;
        }

        private Patient GetPatient(Guid patientId)
        { 
            PatientSearchCriteria criteria = new PatientSearchCriteria();
            criteria.Id = patientId;
            Patient patient = directoryService.FindPatient(criteria)[0];
            return patient;
        }

        private void lstAppointments_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            Appointment appointment = e.Item.Tag as Appointment;
            if (appointment != null)
            {
                this._selectedAppointment = appointment;

                OnSelectionChanged(this, EventArgs.Empty);
            }
        }

        protected void OnSelectionChanged(object sender, EventArgs e)
        {
            if (SelectionChanged != null)
                SelectionChanged(sender, e);
        }

        public override void Refresh()
        {
            base.Refresh();
            GetAppointmentList();
        }        
    }
}
