//===============================================================================
// Microsoft Aspiring Software Architects Program
// E-Appointments - Case Study Implementation
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.Security.Principal;
using WFExtensions;
using EAppointments.BMS.Services;
using EAppointments.BMS.Services.Contracts;
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Data;

namespace BMS.UnitTests.ServiceInterface
{
    public class BaseHosting
    {

        private static BaseHosting _baseHosting;
        private bool isInit = false;
        
        private ServiceHost appointmentServiceHost;
        private ServiceHost providerServiceHost;
        private ServiceHost directoryServiceHost;

        #region Constructor

        private BaseHosting()
        {

        }

        #endregion

        public static BaseHosting GetInstance()
        {
            if (_baseHosting == null)
            {
                _baseHosting = new BaseHosting();
            }

            return _baseHosting;
        }

        public void Init()
        {
            if (!isInit)
            {
                appointmentServiceHost = new WFServiceHost(typeof(EAppointments.BMS.Services.AppointmentService), new Uri(EAppointments.BMS.UnitTests.Properties.Settings.Default.AppointmentServiceAddress));
                appointmentServiceHost.Open();

                providerServiceHost = new ServiceHost(typeof(EAppointments.BMS.Services.ProviderService), new Uri(EAppointments.BMS.UnitTests.Properties.Settings.Default.ProviderServiceAddress));
                providerServiceHost.Open();

                directoryServiceHost = new ServiceHost(typeof(EAppointments.BMS.Services.DirectoryService), new Uri(EAppointments.BMS.UnitTests.Properties.Settings.Default.DirectoryService));
                directoryServiceHost.Open();
                
                isInit = true;
            }
        }
        public void CleanUp()
        {
            if (isInit)
            {
                if (appointmentServiceHost.State == CommunicationState.Opened)
                {
                    appointmentServiceHost.Close();
                }
                if (providerServiceHost.State == CommunicationState.Opened)
                {
                    providerServiceHost.Close();
                }
                if (directoryServiceHost.State == CommunicationState.Opened)
                {
                    directoryServiceHost.Close();
                }
            }
        }

        public ChannelFactory<TContract> GetFactory<TContract, TService>(string userName)
        {
            ChannelFactory<TContract> factory;

            // provide client credentials to the channel
            ClientCredentials clientCredentials = new ClientCredentials();
            clientCredentials.UserName.UserName = userName;
            clientCredentials.UserName.Password = "password";

            // Create a channel
            factory = new ChannelFactory<TContract>(typeof(TService).ToString() + "_Client");
            factory.Endpoint.Behaviors.Remove(typeof(ClientCredentials));
            factory.Endpoint.Behaviors.Add(clientCredentials);
            return factory;
        }

        public void CloseFactory(IChannelFactory factory)
        {
            if (factory.State == CommunicationState.Opened)
                factory.Close();
        }
    }
}
