//===============================================================================
// Microsoft Aspiring Software Architects Program
// E-Appointments - Case Study Implementation
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Text;
using System.Collections.Generic;
using System.ServiceModel;
using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EAppointments.BMS.Services;
using EAppointments.BMS.Services.Contracts;
using System.ServiceModel.Security;
using System.ServiceModel.Description;
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Data;

namespace BMS.UnitTests.ServiceInterface
{
    [TestClass]
    public class BookAppointmentsTestFixture
    {

        private string userName = "bmsadmin";   // Default UserName
        private Guid loggedInUserId = new Guid("646DE0FA-455F-4687-8EE3-2528413AB72B");

        private int numOfPendingAppointments = 5;
        private int numOfBookedAppointments = 2;
        private int numOfApprovedAppointments = 2;
        private int numOfCancelledAppointments = 2;
        private int numOfRejectedAppointments = 2;


        [ClassInitialize]
        public static void ClassInitialize(TestContext context)
        {
            BaseHosting.GetInstance().Init();

        }

        [ClassCleanup]
        public static void CleanUp()
        {
            BaseHosting.GetInstance().CleanUp();
        }

        [TestMethod]
        public void PopulateDatabase()
        {
            ChannelFactory<IAppointmentService> appointmentFactory = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("bmsadmin");
            ChannelFactory<IProviderService> providerFactory = BaseHosting.GetInstance().GetFactory<IProviderService, ProviderService>("bmsadmin");
            ChannelFactory<IDirectoryService> directoryFactory = BaseHosting.GetInstance().GetFactory<IDirectoryService, DirectoryService>("bmsadmin");

            IAppointmentService appointmentProxy = appointmentFactory.CreateChannel();
            IProviderService providerProxy = providerFactory.CreateChannel();
            IDirectoryService directoryProxy = directoryFactory.CreateChannel();

            try
            {
                // Clean up Database
                Database db = DatabaseFactory.CreateDatabase();

                #region Comment out this code to Keep some sample data in the database

                db.ExecuteNonQuery(CommandType.Text, "delete from Appointment");
                db.ExecuteNonQuery(CommandType.Text, "update Slot Set UBRN=null, Status = 1");

                #endregion

                #region Create Pending Appointments

                // Create and Save Pending Appointments into the Database.
                for (int count = 0; count < numOfPendingAppointments; count++)
                {
                    Appointment appointment = new Appointment();

                    ReferrerSearchCriteria referrerCriteria = new ReferrerSearchCriteria();
                    referrerCriteria.FirstName = "sanjiv";
                    referrerCriteria.LastName = "chawla";
                    Referrer referrer = directoryProxy.FindReferrer(referrerCriteria)[0];
                    appointment.Referrer = referrer;

                    PatientSearchCriteria patientCriteria = new PatientSearchCriteria();
                    patientCriteria.ReferrerId = referrer.Id;
                    patientCriteria.FirstName = "vikram";
                    Patient patient = directoryProxy.FindPatient(patientCriteria)[0];
                    appointment.Patient = patient;

                    ClinicTypeCollection clinicTypes = directoryProxy.FindClinicType(new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45"));
                    appointment.ClinicType = clinicTypes.Find(delegate(ClinicType c)
                                                {
                                                    return c.Id == new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                                                }
                                              );

                    ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria();
                    providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                    providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");
                    providerCriteria.Id = new Guid("B016A0CB-E552-4683-A0A1-850340D25A18");
                    Provider provider = providerProxy.FindProvider(providerCriteria)[0];
                    appointment.Provider = provider;

                    Appointment savedAppointment = appointmentProxy.Save(appointment);
                }

                #endregion

                #region Create Booked Appointments

                // Create and Save Booked Appointments into the Database
                for (int count = 0; count < numOfBookedAppointments; count++)
                {
                    // Create a New Appointment
                    Appointment appointment = new Appointment();

                    ReferrerSearchCriteria referrerCriteria = new ReferrerSearchCriteria();
                    referrerCriteria.FirstName = "sanjiv";
                    referrerCriteria.LastName = "chawla";
                    Referrer referrer = directoryProxy.FindReferrer(referrerCriteria)[0];
                    appointment.Referrer = referrer;

                    PatientSearchCriteria patientCriteria = new PatientSearchCriteria();
                    patientCriteria.ReferrerId = referrer.Id;
                    patientCriteria.FirstName = "vikram";
                    Patient patient = directoryProxy.FindPatient(patientCriteria)[0];
                    appointment.Patient = patient;

                    ClinicTypeCollection clinicTypes = directoryProxy.FindClinicType(new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45"));
                    appointment.ClinicType = clinicTypes.Find(delegate(ClinicType c)
                                                {
                                                    return c.Id == new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                                                }
                                              );

                    ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria();
                    providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                    providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");
                    providerCriteria.Id = new Guid("B016A0CB-E552-4683-A0A1-850340D25A18");
                    Provider provider = providerProxy.FindProvider(providerCriteria)[0];
                    appointment.Provider = provider;

                    Appointment savedAppointment = appointmentProxy.Save(appointment);

                    SlotSearchCriteria slotCriteria = new SlotSearchCriteria();
                    slotCriteria.Status = (int)SlotStatus.Available;
                    slotCriteria.ProviderId = savedAppointment.Provider.Id;
                    slotCriteria.ClinicTypeId = savedAppointment.ClinicType.Id;

                    Slot slot = providerProxy.FindSlot(slotCriteria)[0];

                    appointmentProxy.Book(savedAppointment.Ubrn, slot.Id);
                    

                }

                #endregion

                #region Create Approved Appointments

                // Create and Approve Appointments
                for (int count = 0; count < numOfApprovedAppointments; count++)
                {

                    // Login as BMSAdmin
                    appointmentFactory.Close();
                    appointmentProxy = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("bmsadmin").CreateChannel();


                    // Create a New Appointment
                    Appointment appointment = new Appointment();

                    ReferrerSearchCriteria referrerCriteria = new ReferrerSearchCriteria();
                    referrerCriteria.FirstName = "sanjiv";
                    referrerCriteria.LastName = "chawla";
                    Referrer referrer = directoryProxy.FindReferrer(referrerCriteria)[0];
                    appointment.Referrer = referrer;

                    PatientSearchCriteria patientCriteria = new PatientSearchCriteria();
                    patientCriteria.ReferrerId = referrer.Id;
                    patientCriteria.FirstName = "vikram";
                    Patient patient = directoryProxy.FindPatient(patientCriteria)[0];
                    appointment.Patient = patient;

                    ClinicTypeCollection clinicTypes = directoryProxy.FindClinicType(new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45"));
                    appointment.ClinicType = clinicTypes.Find(delegate(ClinicType c)
                                                {
                                                    return c.Id == new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                                                }
                                              );

                    ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria();
                    providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                    providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");
                    providerCriteria.Id = new Guid("B016A0CB-E552-4683-A0A1-850340D25A18");
                    Provider provider = providerProxy.FindProvider(providerCriteria)[0];
                    appointment.Provider = provider;

                    Appointment savedAppointment = appointmentProxy.Save(appointment);
                    

                    SlotSearchCriteria slotCriteria = new SlotSearchCriteria();
                    slotCriteria.Status = (int)SlotStatus.Available;
                    slotCriteria.ProviderId = savedAppointment.Provider.Id;
                    slotCriteria.ClinicTypeId = savedAppointment.ClinicType.Id;

                    Slot slot = providerProxy.FindSlot(slotCriteria)[0];

                    appointmentProxy.Book(savedAppointment.Ubrn, slot.Id);
                    

                    // Login as Provider Clinician to Approve Appointment.
                    appointmentFactory.Close();
                    appointmentProxy = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("jehangir").CreateChannel();
                    appointmentProxy.Approve(savedAppointment.Ubrn);
                    

                }

                #endregion

                #region Create Rejected Appointments

                // Create and Reject Appointments in the database
                for (int count = 0; count < numOfRejectedAppointments; count++)
                {

                    // Login as BMSAdmin
                    appointmentFactory.Close();
                    appointmentProxy = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("bmsadmin").CreateChannel();

                    // Create a New Appointment
                    Appointment appointment = new Appointment();

                    ReferrerSearchCriteria referrerCriteria = new ReferrerSearchCriteria();
                    referrerCriteria.FirstName = "sanjiv";
                    referrerCriteria.LastName = "chawla";
                    Referrer referrer = directoryProxy.FindReferrer(referrerCriteria)[0];
                    appointment.Referrer = referrer;

                    PatientSearchCriteria patientCriteria = new PatientSearchCriteria();
                    patientCriteria.ReferrerId = referrer.Id;
                    patientCriteria.FirstName = "vikram";
                    Patient patient = directoryProxy.FindPatient(patientCriteria)[0];
                    appointment.Patient = patient;

                    ClinicTypeCollection clinicTypes = directoryProxy.FindClinicType(new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45"));
                    appointment.ClinicType = clinicTypes.Find(delegate(ClinicType c)
                                                {
                                                    return c.Id == new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                                                }
                                              );

                    ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria();
                    providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                    providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");
                    providerCriteria.Id = new Guid("B016A0CB-E552-4683-A0A1-850340D25A18");
                    Provider provider = providerProxy.FindProvider(providerCriteria)[0];
                    appointment.Provider = provider;

                    Appointment savedAppointment = appointmentProxy.Save(appointment);

                    SlotSearchCriteria slotCriteria = new SlotSearchCriteria();
                    slotCriteria.Status = (int)SlotStatus.Available;
                    slotCriteria.ProviderId = savedAppointment.Provider.Id;
                    slotCriteria.ClinicTypeId = savedAppointment.ClinicType.Id;

                    Slot slot = providerProxy.FindSlot(slotCriteria)[0];

                    appointmentProxy.Book(savedAppointment.Ubrn, slot.Id);
                    

                    // Login as Provider Clinician to Approve Appointment.
                    appointmentFactory.Close();
                    appointmentProxy = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("jehangir").CreateChannel();
                    appointmentProxy.Reject(savedAppointment.Ubrn);
                    

                }

                #endregion

                #region Create Cancelled Appointments

                // Create and Cancel Appointments in the Database
                for (int count = 0; count < numOfCancelledAppointments; count++)
                {
                    // Login as BMSAdmin
                    appointmentFactory.Close();
                    appointmentProxy = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("bmsadmin").CreateChannel();

                    // Create a New Appointment
                    Appointment appointment = new Appointment();

                    ReferrerSearchCriteria referrerCriteria = new ReferrerSearchCriteria();
                    referrerCriteria.FirstName = "sanjiv";
                    referrerCriteria.LastName = "chawla";
                    Referrer referrer = directoryProxy.FindReferrer(referrerCriteria)[0];
                    appointment.Referrer = referrer;

                    PatientSearchCriteria patientCriteria = new PatientSearchCriteria();
                    patientCriteria.ReferrerId = referrer.Id;
                    patientCriteria.FirstName = "vikram";
                    Patient patient = directoryProxy.FindPatient(patientCriteria)[0];
                    appointment.Patient = patient;

                    ClinicTypeCollection clinicTypes = directoryProxy.FindClinicType(new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45"));
                    appointment.ClinicType = clinicTypes.Find(delegate(ClinicType c)
                                                {
                                                    return c.Id == new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                                                }
                                              );

                    ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria();
                    providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                    providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");
                    providerCriteria.Id = new Guid("B016A0CB-E552-4683-A0A1-850340D25A18");
                    Provider provider = providerProxy.FindProvider(providerCriteria)[0];
                    appointment.Provider = provider;

                    Appointment savedAppointment = appointmentProxy.Save(appointment);

                    SlotSearchCriteria slotCriteria = new SlotSearchCriteria();
                    slotCriteria.Status = (int)SlotStatus.Available;
                    slotCriteria.ProviderId = savedAppointment.Provider.Id;
                    slotCriteria.ClinicTypeId = savedAppointment.ClinicType.Id;

                    Slot slot = providerProxy.FindSlot(slotCriteria)[0];

                    appointmentProxy.Book(savedAppointment.Ubrn, slot.Id);

                    appointmentProxy.Cancel(savedAppointment.Ubrn, "Unit Testing");

                }

                #endregion

            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(appointmentFactory);
                BaseHosting.GetInstance().CloseFactory(providerFactory);
                BaseHosting.GetInstance().CloseFactory(directoryFactory);
            }

        }
        
        private void BookAppointmentWithBookedSlot(string loginUserName)
        {

            ChannelFactory<IAppointmentService> appointmentFactory = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>(loginUserName);
            ChannelFactory<IProviderService> providerFactory = BaseHosting.GetInstance().GetFactory<IProviderService, ProviderService>(loginUserName);
            ChannelFactory<IDirectoryService> directoryFactory = BaseHosting.GetInstance().GetFactory<IDirectoryService, DirectoryService>(loginUserName);

            IAppointmentService appointmentProxy = appointmentFactory.CreateChannel();
            IProviderService providerProxy = providerFactory.CreateChannel();
            IDirectoryService directoryProxy = directoryFactory.CreateChannel();

            try
            {
                SlotSearchCriteria slotCriteria = new SlotSearchCriteria();
                slotCriteria.Status = (int)SlotStatus.Booked;

                Slot slot = providerProxy.FindSlot(slotCriteria)[0];

                Guid slotId = slot.Id;

                AppointmentSearchCriteria criteria = new AppointmentSearchCriteria();
                criteria.Status = (int)AppointmentStatus.Pending;
                AppointmentCollection appointments = appointmentProxy.Find(criteria);

                Appointment foundAppointment = appointments.Find(
                                                delegate(Appointment a)
                                                {
                                                    return (a.WorkflowId != null && a.Status == AppointmentStatus.Pending);
                                                }
                                                );

                appointmentProxy.Book(foundAppointment.Ubrn, slotId);

            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(appointmentFactory);
                BaseHosting.GetInstance().CloseFactory(providerFactory);
                BaseHosting.GetInstance().CloseFactory(directoryFactory);
            }


        }

        /// <summary>
        /// A Referrer tries to create an Appointment with an already booked slot
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(FaultException<AppointmentServiceFault>))]
        public void BookAppointmentWithBookedSlotReferrer()
        {
            BookAppointmentWithBookedSlot("sanjiv");

        }

        /// <summary>
        /// A Patient tries to create an Appointment with an already booked slot
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(FaultException<AppointmentServiceFault>))]
        public void BookAppointmentWithBookedSlotPatient()
        {
            BookAppointmentWithBookedSlot("vikram");

        }

        /// <summary>
        /// Try to create an Appointment with a slot that is available
        /// </summary>
        [TestMethod]
        public void BookAppointmentWithAvailableSlot()
        {
            ChannelFactory<IAppointmentService> appointmentFactory = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>(userName);
            ChannelFactory<IProviderService> providerFactory = BaseHosting.GetInstance().GetFactory<IProviderService, ProviderService>(userName);
            ChannelFactory<IDirectoryService> directoryFactory = BaseHosting.GetInstance().GetFactory<IDirectoryService, DirectoryService>(userName);

            IAppointmentService appointmentProxy = appointmentFactory.CreateChannel();
            IProviderService providerProxy = providerFactory.CreateChannel();
            IDirectoryService directoryProxy = directoryFactory.CreateChannel();
            try
            {
                // Create a New Appointment
                Appointment appointment = new Appointment();

                ReferrerSearchCriteria referrerCriteria = new ReferrerSearchCriteria();
                referrerCriteria.FirstName = "Rahul";
                referrerCriteria.LastName = "Khedekar";
                Referrer referrer = directoryProxy.FindReferrer(referrerCriteria)[0];
                appointment.Referrer = referrer;

                PatientSearchCriteria patientCriteria = new PatientSearchCriteria();
                patientCriteria.ReferrerId = referrer.Id;
                Patient patient = directoryProxy.FindPatient(patientCriteria)[0];
                appointment.Patient = patient;

                ClinicTypeCollection clinicTypes = directoryProxy.FindClinicType(new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45"));
                appointment.ClinicType = clinicTypes.Find(delegate(ClinicType c)
                                            {
                                                return c.Id == new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                                            }
                                          );

                ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria();
                providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");
                providerCriteria.Id = new Guid("B016A0CB-E552-4683-A0A1-850340D25A18");
                Provider provider = providerProxy.FindProvider(providerCriteria)[0];
                appointment.Provider = provider;

                Appointment savedAppointment = appointmentProxy.Save(appointment);
                

                AppointmentSearchCriteria appointmentCriteria = new AppointmentSearchCriteria();
                appointmentCriteria.Ubrn = savedAppointment.Ubrn;

                Appointment foundAppointment = appointmentProxy.Find(appointmentCriteria)[0];

                Assert.AreEqual(foundAppointment.Patient.Id, patient.Id);
                Assert.AreEqual(foundAppointment.Provider.Id, provider.Id);

                SlotSearchCriteria slotCriteria = new SlotSearchCriteria();
                slotCriteria.Status = (int)SlotStatus.Available;
                slotCriteria.ProviderId = foundAppointment.Provider.Id;
                slotCriteria.ClinicTypeId = foundAppointment.ClinicType.Id;

                Slot slot = providerProxy.FindSlot(slotCriteria)[0];

                appointmentProxy.Book(foundAppointment.Ubrn, slot.Id);

                foundAppointment = appointmentProxy.Find(appointmentCriteria)[0];
                Assert.AreEqual(foundAppointment.Status, AppointmentStatus.Booked);
            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(appointmentFactory);
                BaseHosting.GetInstance().CloseFactory(providerFactory);
                BaseHosting.GetInstance().CloseFactory(directoryFactory);
            }
        }

        [TestMethod]
        public void BookExistingAppointment()
        {
            ChannelFactory<IAppointmentService> appointmentFactory = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>(userName);
            ChannelFactory<IProviderService> providerFactory = BaseHosting.GetInstance().GetFactory<IProviderService, ProviderService>(userName);
            ChannelFactory<IDirectoryService> directoryFactory = BaseHosting.GetInstance().GetFactory<IDirectoryService, DirectoryService>(userName);

            IAppointmentService appointmentProxy = appointmentFactory.CreateChannel();
            IProviderService providerProxy = providerFactory.CreateChannel();
            IDirectoryService directoryProxy = directoryFactory.CreateChannel();

            try
            {
                AppointmentSearchCriteria appointmentCriteria = new AppointmentSearchCriteria();
                AppointmentCollection appointments = appointmentProxy.Find(appointmentCriteria);

                Appointment foundAppointment = appointments.Find(
                                                delegate(Appointment a)
                                                {
                                                    return (a.WorkflowId != null && a.Status == AppointmentStatus.Pending);
                                                }
                                                );

                SlotSearchCriteria slotCriteria = new SlotSearchCriteria();
                slotCriteria.Status = (int)SlotStatus.Available;
                slotCriteria.ProviderId = foundAppointment.Provider.Id;
                slotCriteria.ClinicTypeId = foundAppointment.ClinicType.Id;

                Slot slot = providerProxy.FindSlot(slotCriteria)[0];

                appointmentProxy.Book(foundAppointment.Ubrn, slot.Id);
                

                foundAppointment = appointmentProxy.Find(appointmentCriteria)[0];
                Assert.AreEqual(foundAppointment.Status, AppointmentStatus.Booked);
            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(appointmentFactory);
                BaseHosting.GetInstance().CloseFactory(providerFactory);
                BaseHosting.GetInstance().CloseFactory(directoryFactory);
            }
        }
    }
}
