//===============================================================================
// Microsoft Aspiring Software Architects Program
// E-Appointments - Case Study Implementation
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Text;
using System.Collections.Generic;
using System.ServiceModel;
using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EAppointments.BMS.Services;
using EAppointments.BMS.Services.Contracts;
using System.ServiceModel.Security;
using System.ServiceModel.Description;

namespace BMS.UnitTests.ServiceInterface
{
    [TestClass]
    public class CancelAppointmentsTestFixture
    {
        private string userName = "bmsadmin";   // Default UserName
        private Guid loggedInUserId = new Guid("646DE0FA-455F-4687-8EE3-2528413AB72B");

        [ClassInitialize]
        public static void ClassInitialize(TestContext context)
        {
            BaseHosting.GetInstance().Init();

        }

        [ClassCleanup]
        public static void CleanUp()
        {
            BaseHosting.GetInstance().CleanUp();
        }
        
        /// <summary>
        /// Try to cancel a Cancelled Appointment
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(FaultException<AppointmentServiceFault>))]
        public void CancelCancelledAppointment()
        {
            ChannelFactory<IAppointmentService> appointmentFactory = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>(userName);
            IAppointmentService appointmentProxy = appointmentFactory.CreateChannel();
            try
            {

                AppointmentSearchCriteria appointmentCriteria = new AppointmentSearchCriteria();
                appointmentCriteria.Status = (int)AppointmentStatus.Cancelled;
                AppointmentCollection appointments = appointmentProxy.Find(appointmentCriteria);

                Appointment foundAppointment = appointments.Find(
                                                delegate(Appointment a)
                                                {
                                                    return (a.WorkflowId != null && a.Status == AppointmentStatus.Cancelled);
                                                }
                                                );

                appointmentProxy.Cancel(foundAppointment.Ubrn, "Cancel Cancelled Appointment Test");
            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(appointmentFactory);
            }
        }

        /// <summary>
        /// Try to cancel a Rejected Appointment
        /// </summary>
        [TestMethod]
        public void CancelRejectedAppointment()
        {
            ChannelFactory<IAppointmentService> appointmentFactory = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("sanjiv");
            IAppointmentService appointmentProxy = appointmentFactory.CreateChannel();
            try
            {
                AppointmentSearchCriteria appointmentCriteria = new AppointmentSearchCriteria();
                appointmentCriteria.Status = (int)AppointmentStatus.Rejected;
                AppointmentCollection appointments = appointmentProxy.Find(appointmentCriteria);

                Appointment foundAppointment = appointments.Find(
                                                delegate(Appointment a)
                                                {
                                                    return (a.WorkflowId != null && a.Status == AppointmentStatus.Rejected);
                                                }
                                                );

                appointmentProxy.Cancel(foundAppointment.Ubrn, "Cancel Rejected Appointment Test");
            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(appointmentFactory);
            }
        }

        /// <summary>
        /// Cancel an Approved Appointment
        /// </summary>
        [TestMethod]
        public void CancelApprovedAppointment()
        {
            ChannelFactory<IAppointmentService> appointmentFactory = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>(userName);
            ChannelFactory<IProviderService> providerFactory = BaseHosting.GetInstance().GetFactory<IProviderService, ProviderService>(userName);
            ChannelFactory<IDirectoryService> directoryFactory = BaseHosting.GetInstance().GetFactory<IDirectoryService, DirectoryService>(userName);

            IAppointmentService appointmentProxy = appointmentFactory.CreateChannel();
            IProviderService providerProxy = providerFactory.CreateChannel();
            IDirectoryService directoryProxy = directoryFactory.CreateChannel();
            try
            {
                // Create a New Appointment
                Appointment appointment = new Appointment();

                ReferrerSearchCriteria referrerCriteria = new ReferrerSearchCriteria();
                referrerCriteria.FirstName = "Rahul";
                referrerCriteria.LastName = "Khedekar";
                Referrer referrer = directoryProxy.FindReferrer(referrerCriteria)[0];
                appointment.Referrer = referrer;

                PatientSearchCriteria patientCriteria = new PatientSearchCriteria();
                patientCriteria.ReferrerId = referrer.Id;
                Patient patient = directoryProxy.FindPatient(patientCriteria)[0];
                appointment.Patient = patient;

                ClinicTypeCollection clinicTypes = directoryProxy.FindClinicType(new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45"));
                appointment.ClinicType = clinicTypes.Find(delegate(ClinicType c)
                                            {
                                                return c.Id == new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                                            }
                                          );

                ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria();
                providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
                providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");
                providerCriteria.Id = new Guid("B016A0CB-E552-4683-A0A1-850340D25A18");
                Provider provider = providerProxy.FindProvider(providerCriteria)[0];
                appointment.Provider = provider;

                SlotSearchCriteria slotCriteria = new SlotSearchCriteria();
                slotCriteria.ProviderId = provider.Id;
                slotCriteria.Status = (int)SlotStatus.Available;
                Slot slot = providerProxy.FindSlot(slotCriteria)[0];
                appointment.Slot = slot;

                appointment.Status = AppointmentStatus.Pending;

                Guid slotId = (Guid)appointment.Slot.Id;
                Appointment savedAppointment = appointmentProxy.Save(appointment);
                int ubrn = savedAppointment.Ubrn;
                appointmentProxy.Book(savedAppointment.Ubrn, slotId);
                

                appointmentFactory.Close();
                appointmentProxy = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("jehangir").CreateChannel();
                appointmentProxy.Approve(savedAppointment.Ubrn);
                

                string cancellationReason = "Cancel Approved Appointment Test";
                appointmentFactory.Close();
                appointmentProxy = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("jdoe").CreateChannel();
                appointmentProxy.Cancel(ubrn, cancellationReason);
                

                AppointmentSearchCriteria appointmentCriteria = new AppointmentSearchCriteria();
                appointmentCriteria.Status = (int)AppointmentStatus.Cancelled;
                appointmentCriteria.Ubrn = ubrn;
                Appointment retrievedAppointment = appointmentProxy.Find(appointmentCriteria)[0];

                // Check if Appointment has been cancelled
                Assert.AreEqual(retrievedAppointment.Status, AppointmentStatus.Cancelled);

                loggedInUserId = new Guid("41F3E9F3-F21C-45D3-B935-9ED87459DB0F");
                Assert.AreEqual(retrievedAppointment.CancelledById, loggedInUserId);
                Assert.AreEqual(retrievedAppointment.CancellationReason, cancellationReason);
                Assert.IsNull(retrievedAppointment.Slot);

                // Delete the Appointment
                appointmentFactory.Close();
                appointmentProxy = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>("jdoe").CreateChannel();
                appointmentProxy.Delete(retrievedAppointment.Ubrn);
                
            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(appointmentFactory);
                BaseHosting.GetInstance().CloseFactory(providerFactory);
                BaseHosting.GetInstance().CloseFactory(directoryFactory);
            }
        }

        /// <summary>
        /// Cancel an Approved Appointment without providing a Cancellation Reason. (Cancellation Reason is Mandatory)
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(FaultException<AppointmentServiceFault>))]
        public void CancelApprovedAppointmentWithoutReason()
        {
            ChannelFactory<IAppointmentService> appointmentFactory = BaseHosting.GetInstance().GetFactory<IAppointmentService, AppointmentService>(userName);

            IAppointmentService appointmentProxy = appointmentFactory.CreateChannel();
            try
            {
                AppointmentSearchCriteria appointmentCriteria = new AppointmentSearchCriteria();
                appointmentCriteria.Status = (int)AppointmentStatus.Approved;
                AppointmentCollection appointments = appointmentProxy.Find(appointmentCriteria);

                Appointment foundAppointment = appointments.Find(
                                                delegate(Appointment a)
                                                {
                                                    return (a.WorkflowId != null && a.Status == AppointmentStatus.Approved);
                                                }
                                                );

                appointmentProxy.Cancel(foundAppointment.Ubrn, null);

            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(appointmentFactory);
            }
        }

    }
}
