//===============================================================================
// Microsoft Aspiring Software Architects Program
// E-Appointments - Case Study Implementation
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Text;
using System.Collections.Generic;
using System.ServiceModel;
using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EAppointments.BMS.Services;
using EAppointments.BMS.Services.Contracts;
using System.ServiceModel.Security;

namespace BMS.UnitTests.ServiceInterface
{
    [TestClass]
    public class ProviderServiceTestFixture
    {

        private string userName = "bmsadmin";   // Default UserName
        private Guid loggedInUserId = new Guid("646DE0FA-455F-4687-8EE3-2528413AB72B");

        [ClassInitialize]
        public static void ClassInitialize(TestContext context)
        {
            BaseHosting.GetInstance().Init();
        }

        [ClassCleanup]
        public static void CleanUp()
        {
            BaseHosting.GetInstance().CleanUp();
        }
        
        /// <summary>
        /// Search for Providers on the basis of Keywords
        /// </summary>
        [TestMethod]
        [Ignore]
        public void SearchServiceProviderOnKeywords()
        {
            ChannelFactory<IProviderService> providerFactory = BaseHosting.GetInstance().GetFactory<IProviderService, ProviderService>(userName);
            
            IProviderService providerProxy = providerFactory.CreateChannel();
            
            try
            {

                ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria();

                // Search on Alternative Services
                providerCriteria.KeyWords = "Transport Available";

                List<Provider> providerList = providerProxy.FindProvider(providerCriteria);

                Assert.IsTrue(providerList.Count > 0);

                foreach (Provider provider in providerList)
                {
                    Assert.IsTrue(provider.AlternativeServices.ToLower().Contains(providerCriteria.KeyWords.ToLower()));
                }

                // Search on Conditions Treated
                providerCriteria.KeyWords = "Spinal Disorders";

                providerList = providerProxy.FindProvider(providerCriteria);

                Assert.IsTrue(providerList.Count > 0);

                foreach (Provider provider in providerList)
                {
                    Assert.IsTrue(provider.ConditionsTreated.ToLower().Contains(providerCriteria.KeyWords.ToLower()));
                }

                // Search on Procedures Performed
                providerCriteria.KeyWords = "Physiotherapy";

                providerList = providerProxy.FindProvider(providerCriteria);

                Assert.IsTrue(providerList.Count > 0);

                foreach (Provider provider in providerList)
                {
                    Assert.IsTrue(provider.ProceduresPerformed.ToLower().Contains(providerCriteria.KeyWords.ToLower()));
                }

                // Search on Exclusions
                providerCriteria.KeyWords = "Do not perform MRIs";

                providerList = providerProxy.FindProvider(providerCriteria);

                Assert.IsTrue(providerList.Count > 0);

                foreach (Provider provider in providerList)
                {
                    Assert.IsTrue(provider.Exclusions.ToLower().Contains(providerCriteria.KeyWords.ToLower()));
                }
            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(providerFactory);
            }

        }

        /// <summary>
        /// Search for Providers on Proximity
        /// </summary>
        [TestMethod]
        public void SearchServiceProviderOnProximity()
        {
            ChannelFactory<IProviderService> providerFactory = BaseHosting.GetInstance().GetFactory<IProviderService, ProviderService>(userName);

            IProviderService providerProxy = providerFactory.CreateChannel();

            try
            {
                string zipCode = "411001";
                double withinMiles = (double)1;

                ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria();
                providerCriteria.ZipCode = zipCode;
                providerCriteria.WithinMiles = withinMiles;

                List<Provider> providerList = providerProxy.FindProvider(providerCriteria);
                Assert.IsTrue(providerList.Count > 0);

                foreach (Provider provider in providerList)
                {
                    Assert.IsTrue(provider.Proximity <= 1);
                }
            }
            finally
            {
                BaseHosting.GetInstance().CloseFactory(providerFactory);
            }
        } 

    }
}
