//===============================================================================
// Microsoft Aspiring Software Architects Program
// E-Appointments - Case Study Implementation
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using EAppointments.UI.ServiceAgents.Interfaces;
using EAppointments.UI.ServiceAgents.AppointmentService;
using EAppointments.BMS.Services.Contracts;

namespace EAppointments.UI.ServiceAgents
{
    public class AppointmentServiceAgent : IAppointmentServiceAgent
    {
        private AppointmentServiceClient GetProxy()
        {
            AppointmentServiceClient proxy = new AppointmentServiceClient();
            proxy.ClientCredentials.UserName.UserName = System.Threading.Thread.CurrentPrincipal.Identity.Name;
            proxy.ClientCredentials.UserName.Password = "password";
            return proxy;
        }
      
        #region IAppointmentServiceAgent Members

        public Appointment[] Find(AppointmentSearchCriteria criteria)
        {
            using (AppointmentServiceClient proxy = GetProxy())
            {
                return proxy.Find(criteria).ToArray();
            }
        }

        public Appointment Save(Appointment appointment)
        {
            using (AppointmentServiceClient proxy = GetProxy())
            {
                return proxy.Save(appointment);
            }
        }

        public void Book(int ubrn, Guid slotId)
        {
            using (AppointmentServiceClient proxy = GetProxy())
            {
                proxy.Book(ubrn, slotId);
            }
        }

        public void Cancel(int ubrn, string cancellationReason)
        {
            using (AppointmentServiceClient proxy = GetProxy())
            {
                proxy.Cancel(ubrn, cancellationReason);
            }
        }

        public void Rebook(int ubrn, Guid slotId)
        {
            using (AppointmentServiceClient proxy = GetProxy())
            {
               proxy.Rebook(ubrn, slotId);
            }
        }

        public void Approve(int ubrn)
        {
            using (AppointmentServiceClient proxy = GetProxy())
            {
                proxy.Approve(ubrn);
            }
        }

        public void Reject(int ubrn)
        {
            using (AppointmentServiceClient proxy = GetProxy())
            {
                proxy.Reject(ubrn);
            }
        }

        public void Delete(int ubrn)
        {
            using (AppointmentServiceClient proxy = GetProxy())
            {
                proxy.Delete(ubrn);
            }
        }

        #endregion
    }
}
