//===============================================================================
// Microsoft Aspiring Software Architects Program
// E-Appointments - Case Study Implementation
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using EAppointments.UI.ServiceAgents.DirectoryService;
using EAppointments.UI.ServiceAgents.Interfaces;
using EAppointments.BMS.Services.Contracts;

namespace EAppointments.UI.ServiceAgents
{
    public class DirectoryServiceAgent : IDirectoryServiceAgent
    {
        private DirectoryServiceClient GetProxy()
        {
            DirectoryServiceClient proxy = new DirectoryServiceClient();
            proxy.ClientCredentials.UserName.UserName = System.Threading.Thread.CurrentPrincipal.Identity.Name;
            proxy.ClientCredentials.UserName.Password = "password";
            return proxy;
        }

        #region IDirectoryServiceAgent Members

        public Patient[] Find(PatientSearchCriteria criteria)
        {
            using (DirectoryServiceClient proxy = GetProxy())
            {
                return proxy.FindPatient(criteria).ToArray();
            }
        }

        public Referrer[] Find(ReferrerSearchCriteria criteria)
        {
            using (DirectoryServiceClient proxy = GetProxy())
            {
                return proxy.FindReferrer(criteria).ToArray();
            }
        }

        public Specialty[] FindSpecialty()
        {
            using (DirectoryServiceClient proxy = GetProxy())
            {
                return proxy.FindSpecialty().ToArray();
            }
        }

        public ClinicType[] FindClinicType(Guid specialtyId)
        {
            using (DirectoryServiceClient proxy = GetProxy())
            {
                return proxy.FindClinicType(specialtyId).ToArray();
            }
        }

        #endregion
    }
}
